@@ -21,8 +21,8 @@ function(file, ignoreBlanks = TRUE, handlers = NULL,
            isURL = FALSE, asTree = FALSE, addAttributeNamespaces = FALSE,
            useInternalNodes = FALSE, isSchema = FALSE,
            fullNamespaceInfo = FALSE, encoding = character(),
-           useDotNames = length(grep("^\\.", names(handlers))) > 0,  # will be switched to TRUE in the future.
-           xinclude = TRUE, addFinalizer = TRUE, error = xmlErrorCumulator())
+           useDotNames = length(grep("^\\.", names(handlers))) > 0, 
+           xinclude = TRUE, addFinalizer = TRUE, error = xmlErrorCumulator(), isHTML = FALSE)
 {
   if(length(file) > 1) {
     file = paste(file, collapse = "\n")
@@ -36,6 +36,14 @@ function(file, ignoreBlanks = TRUE, handlers = NULL,
     isURL <- length(grep("^(http|ftp|file)://", file, useBytes = TRUE, perl = TRUE))
 
 
+  if(isHTML) {
+    validate = FALSE
+    getDTD = FALSE
+    isSchema = FALSE
+    docClass = "HTMLInternalDocument"
+  } else
+    docClass = character()
+
   checkHandlerNames(handlers, "DOM")
 
   if(missing(fullNamespaceInfo) && inherits(handlers, "RequiresNamespaceInfo"))
@@ -96,26 +104,22 @@ function(file, ignoreBlanks = TRUE, handlers = NULL,
               as.logical(ignoreBlanks), as.logical(replaceEntities),
               as.logical(asText), as.logical(trim), as.logical(validate), as.logical(getDTD),
               as.logical(isURL), as.logical(addAttributeNamespaces),
-              as.logical(useInternalNodes), FALSE, as.logical(isSchema),
+              as.logical(useInternalNodes), as.logical(isHTML), as.logical(isSchema),
               as.logical(fullNamespaceInfo), as.character(encoding), as.logical(useDotNames),
               xinclude, error, addFinalizer, PACKAGE = "XML")
 
 
-
-#  if(inherits(ans, "XMLParseError"))
-#    stop(ans)
-  
-  if(!missing(handlers) & !as.logical(asTree))
+  if(!missing(handlers) && length(handlers) && !as.logical(asTree))
     return(handlers)
 
   if(!isSchema && length(class(ans)))
-    class(ans) = oldClass(class(ans))
+    class(ans) = c(docClass, oldClass(class(ans)))
 
   if(inherits(ans, "XMLInternalDocument"))
     addDocFinalizer(ans, addFinalizer)
   else if(!getDTD && !isSchema) {
        #??? is this a good idea.
-     class(ans) = oldClass("XMLDocumentContent")
+     class(ans) =  oldClass("XMLDocumentContent")
   } 
 
   ans
@@ -160,6 +164,8 @@ function(obj, ...)
   standardGeneric("getEncoding")
 })
 
+setMethod("getEncoding", "ANY", function(obj, ...) NA)
+
 setMethod("getEncoding", "XMLInternalDocument",
             function(obj, ...) {
               .Call("R_getDocEncoding", obj, PACKAGE = "XML")
@@ -171,8 +177,12 @@ setMethod("getEncoding", "XMLInternalNode",
             })
 
 
-
-
+if(FALSE) {
+setMethod("getEncoding", "XMLInternalDOM",
+            function(obj, ...) {
+               getEncoding(obj)
+            })
+}
 
 xmlValidity =
 function(val = integer(0))
