@@ -287,10 +287,6 @@ RS_XML(ParseTree)(USER_OBJECT_ fileName, USER_OBJECT_ converterFunctions,
      return(R_createXMLDocRef(doc));
   }
 
-#if 0   /* Now garbage collected by R.*/
-  xmlFreeDoc(doc);
-  R_numXMLDocsFreed++;
-#endif
 
   if(!parserSettings.internalNodeReferences) {
      /* Set the class for the document. */
@@ -1486,7 +1482,7 @@ R_getDocEncoding(SEXP r_doc)
 
     encoding = doc->encoding;
     PROTECT(ans = NEW_CHARACTER(1));
-    SET_STRING_ELT(ans, 0, CreateCharSexpWithEncoding(doc->encoding, doc->encoding));
+    SET_STRING_ELT(ans, 0, encoding ? CreateCharSexpWithEncoding(doc->encoding, doc->encoding) : R_NaString);
     UNPROTECT(1);
 
     return(ans);
